# Aug 25, 2016
# Koji Ota


package require Tk
package require TclOO

option add *Font {{Meiryo UI} 9}

###############################################################################
# Utility function
proc native_filename { file } {
	set f2 [file nativename $file]
	regsub -all {\\} $f2 {\\\\} f3
	return $f3
}

set uid 1
proc newid { } {
	set id $::uid
	incr ::uid
	return $id
}

###############################################################################
# data
oo::class create Data {
	variable m_prop
	
	constructor args {
		foreach {name value} $args {
			set m_prop($name) $value
		}
	}
	
	method set { args } {
		foreach {name value} $args {
			set m_prop($name) $value
		}
	}
	
	method get { name } {
		return $m_prop($name)
	}
	
	method exists { name } {
		return [info exists m_prop($name)]
	}
	
	method unset { {name {}} } {
		array unset m_prop $name
	}
	
	method lappend { name value } {
		lappend m_prop($name) $value
	}
}

###############################################################################
# Ko2Config
oo::class create Ko2Config {
	superclass Data
	variable m_prop
	variable m_mapped
	variable m_toplevel
	
	constructor args {
		eval [subst "next $args"]]
		set m_mapped 0
	}
	
	method show { } {
		if !$m_mapped {
			set m_toplevel .config
			toplevel $m_toplevel
			wm protocol $m_toplevel WM_DELETE_WINDOW "[self] hide"
			set m_mapped 1
		}
		wm deiconify $m_toplevel
	}
	
	method hide { } {
		wm withdraw $m_toplevel
	}
}
set config [Ko2Config new]

$config set TEXT.fg black
$config set TEXT.bg #fffacd

###############################################################################
# Ko2NoteBookApp class
oo::class create Ko2NoteBookApp {
	variable m_notebook
	variable m_current
	variable m_views
	variable m_statusbar
	variable m_search
	variable m_calendar
	variable m_findstr
	
	constructor args {
		set m_current {}
		set m_search {}
		set m_calendar {}
		set m_findstr [Ko2FindStr new]
		
		# j[o[
		set mb .mb
		pack [frame $mb] -fill x
		set m .mb.prog.m
		pack [menubutton $mb.prog -text "Program" -menu $m] -side left
		menu $m -tearoff 0
		$m add command -label "Folder" -command "[self] explorer [native_filename [pwd]]"
		$m add separator
		$m add command -label "Config" -command "$::config show"
		$m add separator
		$m add command -label "Quit" -command "exit"
		
		set m .mb.edit.m
		pack [menubutton $mb.edit -text "Edit" -menu $m] -side left
		menu $m -tearoff 0
		$m add command -label "Copy to clipboard with HTML format" -command "[self] clipboard"
		
		set m .mb.view.m
		pack [menubutton $mb.view -text "View" -menu $m] -side left
		menu $m -tearoff 0
		$m add command -label "Search" -command "[self] show search"
		$m add command -label "Calendar" -command "[self] show calendar"
		
		# ^u
		set f .f
		pack [frame $f] -fill both -expand 1
		pack [ttk::notebook $f.nb] -fill both -expand 1
		bind $f.nb <<NotebookTabChanged>> "[self] tabChanged"
		set m_notebook $f.nb
		ttk::notebook::enableTraversal $m_notebook
		set m_views {}
		
		# Xe[^Xo[
		set f .sb
		pack [frame $f] -fill x
		pack [label $f.sb -text "" -anchor w -justify left -height 1] -fill x -expand 1
		set m_statusbar $f.sb
		
		# L[oCh
		bind . <Control-r> "[self] redraw"
		bind . <Control-e> "[self] open"
		bind . <Control-x><Control-s> "[self] save"
		bind . <Control-s> "[self] save"
		bind . <Control-f> "[self] find"
		bind . <Control-\;> "[self] insert_date"
		bind . <Control-c><Control-r> "[self] cache refresh"
		bind . <Control-p><Control-s> "[self] show search"
		bind . <Control-p><Control-c> "[self] show calendar"
	}
	
	# obt@s
	method find { } {
		# Jg̖OAcalendar/search̏ꍇ͉Ȃ
		set name [$m_current name]
		if {$name == "calendar" || $name == "search"} {
			return
		}
		$m_findstr show $m_current
	}
	
	# bZ[W\
	method message { msg } {
		$m_statusbar configure -text $msg
	}
	
	method explorer { file } {
		puts $file
		exec explorer $file &
	}
	
	# JgǗ:XV
	method redraw { } {
		puts "m_current=$m_current"
		$m_current redraw
	}
	
	# JgǗ:Lbg郊Ns
	method clipboard { } {
		$m_current clipboard
	}
	
	# JgǗ:t}
	method insert_date { } {
		set msg [clock format [clock second] -format {[%Y/%m/%d %H:%M:%S] }]
		$m_current insert $msg
	}
	
	# JgǗ:LbV
	method cache { cmd } {
		$m_current cache $cmd
	}
	
	# JgǗ:Lbg郊Ns
	method open { } {
		$m_current exec_near_caret
	}
	
	# JgǗ:ۑ
	method save { } {
		$m_current save
	}
	
	# r[Ǘ: viewIuWFNgǉ
	method add { view } {
		$view create $m_notebook
		$m_notebook select [$view frame]
		set m_current $view
		lappend m_views $view
		puts "m_current=$m_current"
	}
	
	# r[Ǘ: viewviewIuWFNg擾
	method view { name } {
		foreach view $m_views {
			if {$name == [$view name]} {
				return $view
			}
		}
		return {}
	}
	
	# r[Ǘ: frameviewIuWFNg擾
	method frame2view { frame } {
		foreach view $m_views {
			if {$frame == [$view frame]} {
				return $view
			}
		}
		return {}
	}
	
	# r[Ǘ: ă[h
	method reload { viewObj } {
		set name [$viewObj name]
		foreach view $m_views {
			if {$view != $viewObj} {
				if {$name == [$view name]} {
					$view reload $name
				}
			}
		}
	}
	
	# ^uǗ:^uύXCxg
	method tabChanged { } {
		set m_current [[self] frame2view [$m_notebook select]]
		switch [$m_current name] {
			calendar -
			search {
			}
			default {
				$m_findstr load $m_current
			}
		}
		puts "tabChanged $m_current"
	}
	
	# ^uǗ:^uؑ
	method change { name {line {}} } {
		set view [[self] view $name]
		if {$view != {}} {
			$m_notebook select [$view frame]
		}
		set m_current $view
		if {$line != {}} {
			$m_current goto $line
		}
	}
	
	# ^uǗ:^u邩ǂ
	method contains { name } {
		if {[[self] view $name] != {}} {
			return 1
		} else {
			return 0
		}
	}
	
	# ^uǗF^u̕\
	method show { what } {
		switch $what {
			search {
				if {$m_search == {}} {
					set m_search [Ko2NoteViewSearch new search]
					[self] add $m_search
				} else {
					[self] change search
				}
			}
			calendar {
				if {$m_calendar == {}} {
					set m_calendar [Ko2NoteViewCalendar new calendar]
					[self] add $m_calendar
				} else {
					[self] change calendar
				}
			}
		}
	}
}

###############################################################################
# Ko2Search class
oo::class create Ko2FindStr {
	variable m_toplevel
	variable m_managed
	variable m_keyword
	variable m_listbox
	variable m_text
	variable m_timer
	variable m_blinker
	variable m_view
	
	constructor args {
		set m_view {}
		set m_text {}
		set m_managed 0
		set m_timer {}
		set m_listbox {}
	}
	
	method manage { } {
		set m_toplevel .findstr
		toplevel $m_toplevel
		set f .findstr.f
		wm protocol $m_toplevel WM_DELETE_WINDOW "[self] hide" 
		pack [frame $f] -fill both -expand 1
		pack [frame $f.w1] -fill x
		pack [label $f.w1.w1 -text ""] -side left
		pack [entry $f.w1.w2 -textvariable [my varname m_keyword]] -side left
		pack [button $f.w1.w3 -text "" -command "[self] clear"] -side left
		pack [frame $f.w2] -fill both -expand 1
		pack [listbox $f.w2.lb -width 80 -height 10 -yscrollcommand "$f.w2.y set"] -side left -fill both -expand 1
		pack [scrollbar $f.w2.y -command "$f.w2.lb yview"] -side left -fill y
		bind $f.w1.w2 <Key-Return> "[self] find"
		
		set m_listbox $f.w2.lb
		#bind $m_listbox <<ListboxSelect>> "[self] lookAt"
		bind $m_listbox <Double-ButtonPress-1> "[self] lookAt"
		
		set m_managed 1
	}
	
	method show { viewObj } {
		if !$m_managed {
			[self] manage
		} else {
			if {[wm state $m_toplevel] == withdraw} {
				wm deiconify $m_toplevel
			}
		}
		[self] load $viewObj
	}
	
	method load { viewObj } {
		set m_view $viewObj
		set name [$m_view name]
		set m_text [$m_view widget]
		set m_keyword ""
		if $m_managed {
			$m_listbox delete 0 end
			wm title $m_toplevel "Find string for <$name>"
		}
	}
	
	method hide { } {
		wm withdraw $m_toplevel
		if {$m_text != {}} {
			$m_text tag remove findstr 0.0 end
		}
		if {$m_timer != {}} {
			after cancel $m_timer
		}
		set m_timer {}
	}
	
	method clear { } {
		set m_keyword ""
		$m_listbox delete 0 end
		$m_text tag remove findstr 0.0 end
		if {$m_timer != {}} {
			after cancel $m_timer
		}
		set m_timer {}
	}
	
	method find { } {
		puts "find $m_keyword"
		
		$m_listbox delete 0 end
		$m_text tag remove findstr 0.0 end
		
		set cur 1.0
		set lineNo 1
		while 1 {
			set cur [$m_text search -count length $m_keyword $cur end]
			if {$cur == ""} {
				break
			}
			$m_text tag add findstr $cur "$cur + $length char"
			set cur [$m_text index "$cur + $length char"]
			
			set text [$m_text get "$cur linestart" "$cur lineend"]
			$m_listbox insert end "[format %4d $lineNo] : $text"
			incr lineNo
		}
		
		set m_blinker 0
		set m_timer [after 800 "[self] blink"]
	}
	
	method blink { } {
		switch $m_blinker {
			0 {
				$m_text tag configure findstr -borderwidth 1 -relief sunken -background "yellow"
				set m_blinker 1
				set m_timer [after 800 "[self] blink"]
			}
			1 {
				$m_text tag configure findstr -borderwidth 0 -relief flat -background ""
				set m_blinker 0
				set m_timer [after 200 "[self] blink"]
			}
		}
	}
	
	method lookAt { } {
		set item [$m_listbox get [$m_listbox curselection]]
		set fi [split $item :]
		set line [lindex $fi 0]
		regsub -all {(\s*)} $line "" line
		
		#$::app lookAt $file $line
	}
}

###############################################################################
# Ko2CalendarMaker
oo::class create Ko2CalendarMaker {
	method isleapyear { year } {
	    #if {[catch "clock scan 2/29/$year"]} {
	    #    return 0
	    #} else {
	    #    return 1
	    #}
	    
	    set isLeapYear 0
	    if {[expr $year % 4] == 0} {
	    	if {[expr $year % 100] == 0} {
	    		if {[expr $year % 100] == 0} {
	    			set isLeapYear 1
	    		}
	    	} else {
	    		set isLeapYear 1
	    	}
	    }
	    return $isLeapYear
	}
	
	method days { year month } {
		#                 1  2  3  4  5  6  7  8  9  10 11 12
		set days [lindex {31 28 31 30 31 30 31 31 30 31 30 31} [expr $month-1]]
		if {$month == 2} {
			if [[self] isleapyear $year] {
				set days 29
			}
		}
		return $days
	}
	
	method firstday { year month } {
		set d [clock scan [format %4d/%02d/01 $year $month] -format "%Y/%m/%d"]
		set w [clock format $d -format %a]
		#puts "w=$w"
		return [lsearch {"" Sun Mon Tue Wed Thu Fri Sat} $w]
	}
}
Ko2CalendarMaker create calendarMaker

###############################################################################
# Ko2Date
oo::class create Ko2Date {
	superclass Data
	variable m_prop
	variable m_ymd
	
	constructor args {
		[self] today
	}
	
	method today { } {
		set now [clock scan "today 00:00:00"]
		[self] ymd $now
	}
	
	method ymd { dvalue } {
		set ymd [clock format $dvalue -format "%Y %m %d"]
		set y [lindex $ymd 0]
		set m [lindex $ymd 1]
		set d [lindex $ymd 2]
		regsub {0*([0-9]*)} $y {\1} y
		regsub {0*([0-9]*)} $m {\1} m
		regsub {0*([0-9]*)} $d {\1} d
		set m_ymd(y) $y
		set m_ymd(m) $m
		set m_ymd(d) $d
	}
	
	method year { args } {
		if {[llength $args] > 0} {
			set m_ymd(y) [lindex $args 0]
		}
		return $m_ymd(y)
	}
	
	method month { args } {
		if {[llength $args] > 0} {
			set m_ymd(m) [lindex $args 0]
		}
		return $m_ymd(m)
	}
	
	method day { args } {
		if {[llength $args] > 0} {
			set m_ymd(d) [lindex $args 0]
		}
		return $m_ymd(d)
	}
	
	method copy_to { des } {
		foreach name [array names m_prop] {
			$des set $name $m_prop($name)
		}
		$des year  $m_ymd(y)
		$des month $m_ymd(m)
		$des day   $m_ymd(d)
	}
	
	method move { what how } {
		switch $what {
			day {
				switch $how {
					next {
						incr m_ymd(d)
						if {[calendarMaker days $m_ymd(y) $m_ymd(m)] < $m_ymd(d)} {
							set m_ymd(d) 1
							incr m_ymd(m)
							if {$m_ymd(m) > 12} {
								incr m_ymd(y)
							}
						}
					}
					prev {
						incr m_ymd(d) -1
						if {1 > $m_ymd(d)} {
							incr m_ymd(m) -1
							if {$m_ymd(m) < 1} {
								incr m_ymd(y) -1
								set m_ymd(m) 12
							}
							set m_ymd(d) [calendarMaker days $m_ymd(y) $m_ymd(m)]
						}
					}
				}
			}
			month {
				switch $how {
					next {
						incr m_ymd(m)
						if {$m_ymd(m) > 12} {
							set m_ymd(m) 1
							incr m_ymd(y)
						}
					}
					prev {
						incr m_ymd(m) -1
						if {$m_ymd(m) < 1} {
							set m_ymd(m) 12
							incr m_ymd(y) -1
						}
					}
				}
				set m_ymd(d) 1
			}
			year {
				switch $how {
					next {
						incr m_ymd(y)
					}
					prev {
						incr m_ymd(y) -1
					}
				}
			}
		}
	}
}

###############################################################################
# Ko2Calendar class
oo::class create Ko2Calendar {
	variable m_selection
	variable m_today
	variable m_cursor
	variable m_canvas
	variable m_map
	variable m_holiday
	variable m_mark
	variable m_font
	variable m_box
	variable m_uw
	variable m_uh
	variable m_command
	
	constructor args {
		set m_holiday {}
		set m_mark {}
		set m_selection {}
		set m_today [Ko2Date new]
		set m_cursor [Ko2Date new]
		set m_font {{Calibri} 9 }
		set m_command "[self] nop"
	}
	
	method nop { args } {
		puts "Ko2Calendar::nop $args"
	}
	
	method create { p args } {
		eval [subst "canvas $p $args"]
		set m_canvas $p
		bind $m_canvas <Configure> "[self] resize"
		return $p
	}
	
	method resize { } {
		set w [winfo width  $m_canvas]
		set h [winfo height $m_canvas]
		#puts "$w $h"
		[self] rebox $w $h
		[self] redraw
	}
	
	method rebox { w h } {
		set xbase 10
		set ybase 10
		set m_uw [expr ($w-($xbase*2))/8]
		set m_uh [expr ($h-($ybase*2))/8]
		
		$m_canvas delete all
		
		# display day cell
		for {set r 0} {$r < 8} {incr r} {
			for {set c 0} {$c < 8} {incr c} {
				set x1 [expr $xbase+$c*$m_uw+1]
				set y1 [expr $ybase+$r*$m_uh+1]
				set x2 [expr $x1+$m_uw-1]
				set y2 [expr $y1+$m_uh-1]
				$m_canvas create rectangle $x1 $y1 $x2 $y2 -tags "box_${c}_${r}" -outline white -fill #eeeeee
				set m_box($c,$r) [list $x1 $y1]
			}
		}
		# display weekday name
		set dayname { Week Sun Mon Tue Wed Thu Fri Sat }
		for {set c 0} {$c < 8} {incr c} {
			set cx [expr $xbase+($m_uw/2)+$c*$m_uw]
			set cy [expr $ybase+($m_uh/2)+ 1*$m_uh]
			set x1 [expr $cx-($m_uw/2)+1]
			set y1 [expr $cy-($m_uh/2)+1]
			set x2 [expr $x1+$m_uw-1]
			set y2 [expr $y1+$m_uh-1]
			$m_canvas create rectangle $x1 $y1 $x2 $y2 -outline "" -fill "#a0a0a0" -outline white
			$m_canvas create text $cx $cy -text [lindex $dayname $c] -font $m_font -fill white
		}
		
		# year and month
		set p1 $m_box(5,0)
		set p2 $m_box(7,0)
		$m_canvas create rectangle [lindex $p1 0] [lindex $p1 1] [expr [lindex $p2 0]+$m_uw-1] [expr [lindex $p2 1]+$m_uh-1] \
			-tags "year_month" -outline white -fill white; #lightsteelblue
		$m_canvas create text [expr [lindex $p2 0]+$m_uw-2] [expr [lindex $p2 1]+$m_uh-2] \
			-tags "year_month year_month_label" -fill blue -font $m_font -anchor se -justify right 
		$m_canvas bind year_month <ButtonPress-1> "[self] click month"
		
		
		# display buttons
		set xy $m_box(0,0)
		set x [expr [lindex $xy 0]+($m_uw/2)]
		set y [expr [lindex $xy 1]+($m_uh/2)]
		$m_canvas create text $x $y -text  -font $m_font -tags "move_prev"
		$m_canvas bind move_prev <ButtonPress-1> "[self] move prev"
		$m_canvas bind box_1_0 <ButtonPress-1> "[self] move next"
		
		set xy $m_box(1,0)
		set x [expr [lindex $xy 0]+($m_uw/2)]
		set y [expr [lindex $xy 1]+($m_uh/2)]
		$m_canvas create text $x $y -text  -font $m_font -tags "move_next"
		$m_canvas bind move_next <ButtonPress-1> "[self] move next"
		$m_canvas bind box_0_0 <ButtonPress-1> "[self] move next"
		
		set xy $m_box(2,0)
		set x [expr [lindex $xy 0]+($m_uw/2)]
		set y [expr [lindex $xy 1]+($m_uh/2)]
		$m_canvas create text $x $y -text  -font $m_font -tags "move_today"
		$m_canvas bind move_today <ButtonPress-1> "[self] move day"
		$m_canvas bind box_2_0 <ButtonPress-1> "[self] move today"
	}
	
	method map { year month } {
		set fd [calendarMaker firstday $year $month]
		set nd [calendarMaker days $year $month]
		array unset m_map
		set c $fd
		set r 2
		for {set d 1} {$d <= $nd} {incr d} {
			set m_map($d) [list $c $r]
			if {$c == 7} {
				set c 1
				incr r
			} else {
				incr c
			}
		}
	}
	
	method redraw { } {
		$m_canvas delete days
		
		set date(y) [$m_cursor year]
		set date(m) [$m_cursor month]
		
		# tcellΉ
		[self] map $date(y) $date(m)
		
		# N\
		set m_year_month [format %d/%02d $date(y) $date(m)]
		$m_canvas itemconfigure year_month_label -text $m_year_month
		
		# \
		foreach d [array names m_map] {
			set c [lindex $m_map($d) 0]
			set r [lindex $m_map($d) 1]
			set box box_${c}_${r}
			set fg black
			switch $c {
				1 { set fg red  }
				7 { set fg blue }
			}
			if {[$m_today year] == $date(y) && [$m_today month] == $date(m) && [$m_today day] == $d} {
				set fg green
			}
			set x [lindex $m_box($c,$r) 0]
			set y [lindex $m_box($c,$r) 1]
			$m_canvas itemconfigure $box -fill #eeeeee
			$m_canvas create text [expr $x+$m_uw-2] [expr $y+2] \
				-anchor ne -justify right -text $d -font $m_font -fill $fg -tags "days day_${c}_${r}"
			$m_canvas bind day_${c}_${r} <ButtonPress-1> "[self] click day $d"
			$m_canvas bind box_${c}_${r} <ButtonPress-1> "[self] click day $d"
		}
		
		# T\
		set days [calendarMaker days $date(y) $date(m)]
		set w1 [clock format [clock scan "$date(y)/$date(m)/1" -format "%Y/%m/%d"] -format %U]
		set w2 [clock format [clock scan "$date(y)/$date(m)/$days" -format "%Y/%m/%d"] -format %U]
		regsub {0([0-9]+)} $w1 {\1} w1
		regsub {0([0-9]+)} $w2 {\1} w2
		set c 0
		set r 2
		for {set w $w1} {$w <= $w2} {incr w} {
			set box box_${c}_${r}
			set x [expr [lindex $m_box($c,$r) 0]+1]
			set y [expr [lindex $m_box($c,$r) 1]+1]
			set x2 [expr $x+$m_uw-1]
			set y2 [expr $y+$m_uh-1]
			$m_canvas create rectangle $x $y $x2 $y2 -outline "" -fill "#a0a0a0" -tags "days weeks$r"
			$m_canvas create text [expr $x+$m_uw-2] [expr $y+2] \
				-anchor ne -justify right -text $w -font $m_font -fill white -tags "days weeks$r"
			$m_canvas bind weeks$r <ButtonPress-1> "[self] click weeks $w $r"
			incr r
		}
		
		#[self] selection show
		[self] holiday show
		[self] mark show
	}
	
	method move { args } {
		switch [lindex $args 0] {
			today {
				$m_cursor today
			}
			next {
				$m_cursor move month next
			}
			prev {
				$m_cursor move month prev
			}
		}
		[self] redraw
	}
	
	method command { cmd } {
		set m_command $cmd
	}
	
	method click { what args } {
		puts "click $what $args"
		set date(y) [$m_cursor year]
		set date(m) [$m_cursor month]
		switch $what {
			day {
				eval $m_command day $date(y) $date(m) [lindex $args 0]
				puts "eval $m_command day $date(y) $date(m) [lindex $args 0]"
			}
			weeks {
				set w [lindex $args 0]
				set r [lindex $args 1]
				set d1 [$m_canvas itemcget day_1_$r -text]
				set d2 [$m_canvas itemcget day_7_$r -text]
				if {$d2 == ""} {
					set d2 [[self] days $date(y) $date(m)]
				}
				eval $m_command week $w $date(y) $date(m) $d1 $d2
			}
			month {
				eval $m_command month $date(y) $date(m)
			}
		}
	}
	
	method selection { cmd args } {
	}
	
	method holiday { what args } {
		set date(y) [$m_cursor year]
		set date(m) [$m_cursor month]
		switch $what {
			clear {
				set m_holiday {}
			}
			load {
				set file [lindex $args 0]
				if ![file exists $file] {
					return
				}
				set fd [open $file]
				set buf [read $fd]
				close $fd
				
				foreach line [split $buf \n] {
					if {[string length $line] == 0} {
						continue
					}
					switch [lindex $line 0] {
						date {		[self] holiday add date [lindex $line 1] }
						offset {	[self] holiday add offset [lindex $line 1] [lindex $line 2] }
					}
				}
			}
			add {
				switch [lindex $args 0] {
					date {		lappend m_holiday [list date [lindex $args 1]] }
					offset {	lappend m_holiday [list offset [lindex $args 1] [lindex $args 2]] }
				}
			}
			show {
				foreach i $m_holiday {
					set md {}
					switch [lindex $i 0] {
						date {
							set md [split [lindex $i 1] /]
						}
						offset {
							set offset [lindex $i 1]
							set base [lindex $i 2]/01/$date(y)
							set md [clock format [clock scan $offset -base [clock scan $base]] -format "%m %d"]
						}
					}
					if {$md == {}} {
						continue
					}
					set m [lindex $md 0]
					set d [lindex $md 1]
					regsub {0([0-9]*)} $m {\1} m
					regsub {0([0-9]*)} $d {\1} d
					
					if {$m == $date(m)} {
						set rc $m_map($d)
						$m_canvas itemconfigure day_[lindex $rc 0]_[lindex $rc 1] -fill red
					}
				}
			}
		}
	}
	
	method mark { what args } {
		set date(y) [$m_cursor year]
		set date(m) [$m_cursor month]
		switch $what {
			clear {
				set m_mark {}
			}
			add {
				# y m d
				eval [subst "lappend m_mark [list $args]"]
			}
			show {
				foreach i $m_mark {
					set y [lindex $i 0]
					set m [lindex $i 1]
					set d [lindex $i 2]
					if {$date(y) == $y && $date(m) == $m} {
						set rc $m_map($d)
						$m_canvas itemconfigure box_[lindex $rc 0]_[lindex $rc 1] -fill orange
					}
				}
			}
		}
	}
}

###############################################################################
# Ko2Buffer class
oo::class create Ko2Buffer {
	variable m_name
	variable m_observers
	constructor args {
		set m_observers {}
	}
	method name { args } {
		if {[llength $args] > 0} {
			set m_name [lindex $args 0]
		}
		return $m_name
	}
	method add { obj } {
		lappend m_observers $obj
	}
	method remove { obj } {
		set p [lsearch $m_observers $obj]
		if {$p >= 0} {
			set m_observers [lreplace $m_observers $p $p]
		}
	}
	method notify { sender } {
		foreach obj $m_observers {
			if {$obj != $sender} {
				$obj notify
			}
		}
	}
}

###############################################################################
# Ko2Buffer class
oo::class create Ko2BufferManager {
	variable m_buffers
	constructor args {
		set m_buffers {}
	}
	method find { name } {
		foreach buf $m_buffers {
			if {[$buf name] == $name} {
				return $buf
			}
		}
		return {}
	}
	method add { name obj } {
		set buf [[self] find $name]
		if {$buf != {}} {
			$buf add $obj
		} else {
			set buf [Ko2Buffer new]
			$buf name $name
			$buf add $obj
			lappend m_buffers $buf
		}
	}
	method remove { name obj } {
		set buf [[self] find $name]
		if {$buf != {}} {
			$buf remove $obj
		}
	}
	method notify { name obj } {
		set buf [[self] find $name]
		if {$buf != {}} {
			$buf notify $obj
		}
	}
}
Ko2BufferManager create bufferManager

###############################################################################
# Ko2NoteNode class
oo::class create Ko2Tag {
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler

	constructor args {
		set m_handler {}
	}

	method name { } {
		return $m_name
	}
	
	method key { } {
		return $m_key
	}
	
	method fg { args } {
		if {[llength $args] > 0} {
			set m_fg [lindex $args 0]
		}
		return $m_fg
	}
	
	method bg { w } {
		if {[llength $args] > 0} {
			set m_bg [lindex $args 0]
		}
		return $m_bg
	}
	
	method get_clicked_value { w tag x y } {
		set linechar [split [$w index current] .]
		set line [lindex $linechar 0]
		set char [lindex $linechar 1]
		set value ""
		set str "\[\[:alnum:\]\\\s\\\-\\\[\\\]\\\(\\\)\\\.\\\?,_:\\\\/AB\]*"
		foreach { start end } [$w tag ranges $tag] {
			set linechar [split $start .]
			set sline [lindex $linechar 0]
			set schar [lindex $linechar 1]
			set linechar [split $end .]
			set eline [lindex $linechar 0]
			set echar [lindex $linechar 1]
			if {$line == $sline && $char >= $schar && $char <= $echar} {
				set text [$w get $sline.$schar $eline.$echar]
				regexp "$tag:\\s*\\{*($str)\\s*\\}*" $text tmp value
				regsub "\{" $value "" value
				regsub "\}" $value "" value
				break
			}
		}
		puts "value=$value"
		return $value
	}
}

###############################################################################
# Ko2NoteNodeTodo class
oo::class create Ko2TagTodo {
	superclass Ko2Tag
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler
	
	constructor args {
		set m_name TODO
		set m_key TODO:
		set m_fg green
		set m_bg "" ;# royalblue
	}
	
	method load { obj w } {
		$w tag configure TODO -foreground $m_fg -background $m_bg
	} 
}

###############################################################################
# Ko2NoteNodeDone class
oo::class create Ko2TagDone {
	superclass Ko2Tag
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler
	
	constructor args {
		set m_name DONE
		set m_key DONE:
		set m_fg ""
		set m_bg #808080
	}
	
	method load { obj w } {
		$w tag configure DONE -foreground $m_fg -background $m_bg -overstrike 1
	} 
}

###############################################################################
# Ko2NoteNodeWait class
oo::class create Ko2TagWait {
	superclass Ko2Tag
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler
	
	constructor args {
		set m_name WAIT
		set m_key WAIT:
		set m_fg black
		set m_bg orange
	}
	
	method load { obj w } {
		$w tag configure WAIT -foreground $m_fg -background $m_bg
	} 
}

###############################################################################
# Ko2NoteNodeH1 class
oo::class create Ko2TagH1 {
	superclass Ko2Tag
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler
	
	constructor args {
		set m_name H1
		set m_key H1:
		set m_fg white
		set m_bg black
	}
	
	method load { obj w } {
		$w tag configure H1 -foreground $m_fg -background $m_bg
	} 
}

###############################################################################
# Ko2NoteNodeNote class
oo::class create Ko2TagNote {
	superclass Ko2Tag
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler
	
	constructor args {
		set m_name NOTE
		set m_key NOTE:
		set m_fg red
		set m_bg ""
	}
	
	method load { obj w } {
		$w tag configure NOTE -foreground $m_fg -background $m_bg
	} 
}

###############################################################################
# Ko2NoteNodePlan class
oo::class create Ko2TagPlan {
	superclass Ko2Tag
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler
	
	constructor args {
		set m_name PLAN
		set m_key PLAN:
		set m_fg black
		set m_bg lime
	}
	
	method load { obj w } {
		$w tag configure PLAN -foreground $m_fg -background $m_bg
	} 
}

###############################################################################
# Ko2NoteNodeTodo class
oo::class create Ko2TagDate {
	superclass Ko2Tag
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler
	
	constructor args {
		set m_name DATE
		set m_key DATE:
		set m_fg white
		set m_bg royalblue
	}
	
	method load { obj w } {
		$w tag configure DATE -foreground $m_fg -background $m_bg
	} 
}

###############################################################################
# Ko2NoteNodeLink class
oo::class create Ko2TagLink {
	superclass Ko2Tag
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler
	
	constructor args {
		set m_name LINK
		set m_key LINK:
		set m_fg blue
		set m_bg ""
	}
	
	method load { obj w } {
		set m_handler $obj
		$w tag configure LINK -foreground $m_fg -background $m_bg -underline 1
		$w tag bind LINK <ButtonPress-1> "[self] click $w %x %y"
		$w tag bind LINK <Enter> "$obj changeCursor hand2"
		$w tag bind LINK <Leave> "$obj changeCursor xterm"
	}
	
	method click { w x y } {
		# current }[N
		#$m_handler action current
		set value [[self] get_clicked_value $w LINK $x $y]
		[self] action $w $value
	}
	
	method action { w value } {
		puts "Ko2TagLink::action $w $value"
		$::app explorer $value
	}
}

###############################################################################
# Ko2NoteNodeMemo class
oo::class create Ko2TagMemo {
	superclass Ko2Tag
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler
	
	constructor args {
		set m_name MEMO
		set m_key MEMO:
		set m_fg white
		set m_bg brown
	}
	
	method load { obj w } {
		set m_handler $obj
		$w tag configure MEMO -foreground $m_fg -background $m_bg
		$w tag bind MEMO <ButtonPress-1> "[self] click $w %x %y"
		$w tag bind MEMO <Enter> "$obj changeCursor hand2"
		$w tag bind MEMO <Leave> "$obj changeCursor xterm"
	} 
	
	method click { x y } {
		# current }[N
		#$m_handler action current
		set value [[self] get_clicked_value $w MEMO $x $y]
		[self] action $w $value
	}
	
	method action { w value } {
		puts "Ko2TagMemo::action $w $value"
	}
}

###############################################################################
# Ko2NoteNodeNode class
oo::class create Ko2TagNode {
	superclass Ko2Tag
	variable m_name
	variable m_key
	variable m_fg
	variable m_bg
	variable m_handler
	
	constructor args {
		set m_name NODE
		set m_key NODE:
		set m_fg white
		set m_bg brown
	}
	
	method load { obj w } {
		set m_handler $obj
		$w tag configure NODE -foreground $m_fg -background $m_bg
		$w tag bind NODE <ButtonPress-1> "[self] click $w %x %y"
		$w tag bind NODE <Enter> "$obj changeCursor hand2"
		$w tag bind NODE <Leave> "$obj changeCursor xterm"
	}
	
	method click { w x y } {
		# current }[N
		#$m_handler action current
		
		set value [[self] get_clicked_value $w NODE $x $y]
		[self] action $w $value
	}
	
	method action { w value } {
		puts "Ko2TagNode::action $w $value"
		if [$::app contains $value] {
			$::app change $value
		} else {
			$::app add [Ko2NoteViewText new $value]
		}
	}
}

###############################################################################
# Ko2NoteHandler class
oo::class create Ko2TextHandler {
	variable m_name
	variable m_view
	variable m_tags
	variable m_keys
	variable m_pattern
	variable m_names
	variable m_text
	variable m_filename
	
	constructor args {
		set m_name ""
	
		set m_view [lindex $args 0]
	
		set m_filename {}
		
		set m_tags {}
		lappend m_tags [Ko2TagTodo new]
		lappend m_tags [Ko2TagDone new]
		lappend m_tags [Ko2TagWait new]
		lappend m_tags [Ko2TagNote new]
		lappend m_tags [Ko2TagLink new]
		lappend m_tags [Ko2TagNode new]
		lappend m_tags [Ko2TagMemo new]
		lappend m_tags [Ko2TagPlan new]
		lappend m_tags [Ko2TagH1 new]
		lappend m_tags [Ko2TagDate new]
		
		set m_names {}
		set m_keys ""
		set sep ""
		foreach tag $m_tags {
			lappend m_names [$tag name]
			append m_keys $sep [$tag key]
			set sep "|"
		}
		
		#set m_pattern "($m_keys)\\s*((\[\[:alnum:\]:\\\\\]*)|({\\s*\[\[:alnum:\]:\\\\\\s\]*}))"
		set str1 "\[\[:alnum:\]\\\-\\\[\\\]\\\(\\\)\\\.\\\?,_:\\\\/AB\]*"
		set str2 "\[\[:alnum:\]\\\-\\\[\\\]\\\(\\\)\\\.\\\?,_:\\\\/AB\\s*\]*"
		set m_pattern "($m_keys)\\s*(($str1)|({\\s*$str2\\s*}))"
	}
	
	method text { args } {
		if {[llength $args] > 0} {
			set m_text [lindex $args 0]
			
			bind $m_text <KeyRelease-Return> "[self] redraw"
			bind $m_text <<Modified>> "[self] modified"
		}
		return $m_text
	}
	
	method add { tag } {
		lappend m_tags $tag
	}
	
	method get { name } {
		foreach tag $m_tags {
			if {$name == [$tag name]} {
				return $tag
			}
		}
		return {}
	}
	
	method reset { } {
		set names ""
		foreach n $m_names {
			append names " $n"
		}
		$m_text tag delete $names
		
		foreach tag $m_tags {
			$tag load [self] $m_text
		}
	}
	
	method modified { } {
		$::app message "$m_name modified."
	}
	
	method redraw { } {
		[self] reset
		
		foreach r [$m_text search -all -regexp "${m_keys}\s*.*" 1.0] {
			set linechar [split $r .]
			set line [lindex $linechar 0]
			set char [lindex $linechar 1]
			set text [$m_text get $line.$char $line.end]
			set x [regexp $m_pattern $text tmp key value]
			if {$x > 0} {
				foreach tag $m_tags {
					if {$key == [$tag key]} {
						set last [expr $char+[string length $tmp]]
						$m_text tag add [$tag name] $line.$char $line.$last
						#puts "$key $tmp"
					}
				}
			}
		}
	}
	
	method changeCursor { name } {
		$m_text configure -cursor $name
	}
	
	method name { } {
		return $m_name
	}
	
	method load { name } {
		set m_name $name
		set m_filename data/$name.txt
		puts "Ko2TextHandler::load $m_filename"
		[self] reload
	}
	
	method reload { } {
		$m_text delete 0.0 end
		if [file exists $m_filename] {
			set fd [open $m_filename]
			set buf [read $fd]
			close $fd
			$m_text insert end $buf
		}
		$m_text edit modified false
		[self] redraw
		
		bufferManager add [file tail $m_filename] [self]
	}
	
	method save { } {
		puts "save"
		$m_text edit modified false
		set fd [open $m_filename w]
		set buf [$m_text get 0.0 "end - 1 char"]
		puts $fd $buf
		close $fd 
		$::app message "$m_name saved."
		
		bufferManager notify [file tail $m_filename] [self]
	}
	
	method notify { } {
		[self] reload
	}
	
	method goto { line } {
		$m_text see $line.0
	}
	
	method lookat { file line } {
		if {$m_name == $file} {
			[self] goto $line
		} else {
			[self] load $file
			[self] goto $line
		}
	}
	
	method action { position } {
		set current [$m_text index $position]
		set cp [split $current .]
		set line [lindex $cp 0]
		set char [lindex $cp 1]
		puts "position=$position, linechar=$line,$char"
		
		set text [$m_text get $line.0 $line.end]
		set x [regexp $m_pattern $text tmp key value]
		foreach tag $m_tags {
			if {$key == [$tag key]} {
				regsub "\{" $value "" value
				regsub "\}" $value "" value
				$tag action $m_text $value
			}
		}
	}
	
	method clipboard { } {
		puts [$m_text dump 0.0 end]
	}
	
	method insert { msg } {
		$m_text insert end $msg
	}
}

###############################################################################
# Ko2NoteView class
oo::class create Ko2NoteView {
	variable m_name
	variable m_frame
	variable m_notebook
	variable m_initialized
	variable m_keys
	variable m_handler
	variable m_text
	
	constructor args {
		set m_name [lindex $args 0]
		
		set m_handler [Ko2TextHandler new [self]]
		set m_initialized 0
	}
	
	method name { } {
		return $m_name
	}
	
	method frame { } {
		return $m_frame
	}
	
	method exec_near_caret { } {
		$m_handler action insert 
	}
	
	method redraw { } {
		puts "Ko2NoteView"
		$m_handler redraw
	}
	
	method load { name } {
		$m_handler load $name
	}
	
	method reload { } {
		$m_handler reload
	}
	
	method save { } {
		$m_handler save
		
		
		# ȊȎ̃r[XV
		$::app reload [self]
	}
	
	method clipboard { } {
	}
	
	method insert { msg } {
		$m_handler insert $msg
	}
	
	method cache { cmd } {
	}
}

###############################################################################
# Ko2NoteViewText class
oo::class create Ko2NoteViewText {
	superclass Ko2NoteView
	variable m_name
	variable m_frame
	variable m_notebook
	variable m_font
	variable m_handler
	variable m_text
	
	constructor args {
		next $args
		set m_font {{Meiryo UI} 9}
	}
	method create { nb } {
		set m_notebook $nb
		set m_frame $m_notebook.f[newid]
		frame $m_frame
		$m_notebook add $m_frame -text $m_name
		
		set f $m_frame
		text $f.t -width 80 -height 40 \
			-fg [$::config get TEXT.fg] -bg [$::config get TEXT.bg] \
			-wrap none \
			-font $m_font \
			-undo true \
			-tabs "[expr {4 * [font measure $m_font 0]}] left" -tabstyle wordprocessor \
			-xscrollcommand "$f.x set" \
			-yscrollcommand "$f.y set"
		scrollbar $f.x -orient horizontal -command "$f.t xview"
		scrollbar $f.y -orient vertical   -command "$f.t yview"
		grid $f.t -row 0 -column 0 -sticky news
		grid $f.y -row 0 -column 1 -sticky ns
		grid $f.x -row 1 -column 0 -sticky ew
		grid columnconfigure $f 0 -weight 1
		grid rowconfigure $f 0 -weight 1
		set m_text $f.t
		
		#bind $m_text <KeyRelease-Return> "[self] redraw"
		$m_handler text $m_text
		
		# t@C[h
		[self] load $m_name
	}
	
	# text widgetԂ
	method widget { } {
		return $m_text
	}
	
	# wsֈړ
	method goto { line } {
		$m_handler goto $line
	}
	
	#
	method clipboard { } {
		$m_handler clipboard
	}
}

###############################################################################
# Ko2NoteViewSearch class
oo::class create Ko2NoteViewSearch {
	superclass Ko2NoteView
	variable m_name
	variable m_frame
	variable m_font
	variable m_handler
	variable m_text
	variable m_listbox
	variable m_keyword
	
	constructor args {
		next $args
		set m_font {{Meiryo UI} 9}
		set m_keyword ""
	}
	method create { nb } {
		set m_notebook $nb
		set m_frame $m_notebook.f[newid]
		frame $m_frame
		$m_notebook add $m_frame -text $m_name
	
		set f $m_frame
		pack [frame $f.w1] -fill x
		pack [label $f.w1.w1 -text ""] -side left
		pack [entry $f.w1.w2 -textvariable [my varname m_keyword]] -side left
		pack [frame $f.w2] -fill both -expand 1
		pack [listbox $f.w2.lb -width 80 -height 10 -yscrollcommand "$f.w2.y set"] -side left -fill both -expand 1
		pack [scrollbar $f.w2.y -command "$f.w2.lb yview"] -side left -fill y
		bind $f.w1.w2 <Key-Return> "[self] find"
		
		set m_listbox $f.w2.lb
		#bind $m_listbox <<ListboxSelect>> "[self] lookAt"
		bind $m_listbox <Double-ButtonPress-1> "[self] lookAt"
		
		pack [frame $f.w3] -fill both -expand 1
		text $f.w3.t -width 80 \
			-fg [$::config get TEXT.fg] -bg [$::config get TEXT.bg] \
			-wrap none \
			-font $m_font \
			-undo true \
			-tabs "[expr {4 * [font measure $m_font 0]}] left" -tabstyle wordprocessor \
			-xscrollcommand "$f.w3.x set" \
			-yscrollcommand "$f.w3.y set"
		scrollbar $f.w3.x -orient horizontal -command "$f.w3.t xview"
		scrollbar $f.w3.y -orient vertical   -command "$f.w3.t yview"
		grid $f.w3.t -row 0 -column 0 -sticky news
		grid $f.w3.y -row 0 -column 1 -sticky ns
		grid $f.w3.x -row 1 -column 0 -sticky ew
		grid columnconfigure $f.w3 0 -weight 1
		grid rowconfigure $f.w3 0 -weight 1
		set m_text $f.w3.t
		
		#bind $m_text <KeyRelease-Return> "[self] redraw"
		$m_handler text $m_text
	}
	
	method find { } {
		#puts "find $m_keyword"
		
		$m_listbox delete 0 end
		
		foreach f [glob -nocomplain "data/*.txt"] {
			puts $f
			
			set fd [open $f]
			set buf [read $fd]
			close $fd
			
			set lineNo 1
			foreach line [split $buf \n] {
				if {[regexp $m_keyword $line tmp] > 0} {
					$m_listbox insert end [format "%-16s:%4d:%s" [file tail $f] $lineNo [string range $line 0 32]]
				}
				incr lineNo
			}
		}
	}
	
	method lookAt { } {
		set item [$m_listbox get [$m_listbox curselection]]
		set fi [split $item :]
		set file [lindex $fi 0]
		set line [lindex $fi 1]
		regsub -all {(\s*)} $file "" file
		regsub -all {(\s*)} $line "" line
		regsub -all {.txt} $file "" file
		#puts "file=$file, line=$line"
		
		$m_handler lookat $file $line
	}
}

###############################################################################
# Ko2NoteViewCalendar class
oo::class create Ko2NoteViewCalendar {
superclass Ko2NoteView
	variable m_name
	variable m_frame
	variable m_font
	variable m_handler
	variable m_text
	variable m_calendar
	variable m_listbox
	variable m_cache
	
	constructor args {
		next $args
		set m_font {{Meiryo UI} 9}
		set m_calendar [Ko2Calendar new]
		$m_calendar holiday load config/holiday.txt
	}
	
	method create { nb } {
		set m_notebook $nb
		set m_frame $m_notebook.f[newid]
		frame $m_frame
		$m_notebook add $m_frame -text $m_name
		
		set f $m_frame
		pack [frame $f.w1] -fill x
		pack [$m_calendar create $f.w1.c1 -width 250 -height 170] -side left
		pack [frame $f.w1.c2] -fill x
		set mb $f.w1.c2.mb
		pack [frame $mb] -fill x
		set m $mb.cache.m
		pack [menubutton $mb.cache -text "Cache" -menu $m] -side left
		menu $m -tearoff 0
		$m add command -label "Refresh" -command "[self] cache refresh"
		pack [frame $f.w1.c2.f] -fill x
		set p $f.w1.c2.f
		listbox $p.lb -yscrollcommand "$p.y set" -xscrollcommand "$p.x set"
		scrollbar $p.y -command "$p.lb yview" -orient vertical
		scrollbar $p.x -command "$p.lb xview" -orient horizontal
		grid $p.lb -row 0 -column 0 -sticky news
		grid $p.x  -row 1 -column 0 -sticky ew
		grid $p.y  -row 0 -column 1 -sticky ns
		grid columnconfigure $p 0 -weight 1
		grid rowconfigure    $p 0 -weight 1
		bind $f.w1.c2 <Key-Return> "[self] find"
		set m_listbox $p.lb
		#bind $m_listbox <<ListboxSelect>> "[self] lookAt"
		bind $m_listbox <Double-ButtonPress-1> "[self] lookAt"
		$m_calendar command "[self] date"
		
		pack [frame $f.w3] -fill both -expand 1
		text $f.w3.t -width 80 \
			-fg [$::config get TEXT.fg] -bg [$::config get TEXT.bg] \
			-wrap none \
			-font $m_font \
			-undo true \
			-tabs "[expr {4 * [font measure $m_font 0]}] left" -tabstyle wordprocessor \
			-xscrollcommand "$f.w3.x set" \
			-yscrollcommand "$f.w3.y set"
		scrollbar $f.w3.x -orient horizontal -command "$f.w3.t xview"
		scrollbar $f.w3.y -orient vertical   -command "$f.w3.t yview"
		grid $f.w3.t -row 0 -column 0 -sticky news
		grid $f.w3.y -row 0 -column 1 -sticky ns
		grid $f.w3.x -row 1 -column 0 -sticky ew
		grid columnconfigure $f.w3 0 -weight 1
		grid rowconfigure $f.w3 0 -weight 1
		set m_text $f.w3.t
		
		#bind $m_text <KeyRelease-Return> "[self] redraw"
		$m_handler text $m_text
		
		[self] cache load
		[self] cache redraw
	}
	
	method date { args } {
		# day <y> <m> <d>
		# weeks <w> <y> <m> <d1> <d2>
		# month <y> <m>
		#puts "date $args"
		switch [lindex $args 0] {
			day		{ [self] find day   [lindex $args 1] [lindex $args 2] [lindex $args 3] }
			week	{ [self] find weeks [lindex $args 1] [lindex $args 2] [lindex $args 3] [lindex $args 4] [lindex $args 5]}
			month	{ [self] find month [lindex $args 1] [lindex $args 2] }
		}
	}
	
	method find { args } {
		#puts "find $args"
		
		$m_listbox delete 0 end
		
		foreach c $m_cache {
			set found 0
			set y [lindex $c 0]
			set m [lindex $c 1]
			set d [lindex $c 2]
			set f [lindex $c 3]
			set l [lindex $c 4]
			set s [lindex $c 5]
			switch [lindex $args 0] {
				day {
					if {$y == [lindex $args 1] && $m == [lindex $args 2] && $d == [lindex $args 3]} {
						set found 1
					}
				}
				weeks {
					if {$y == [lindex $args 2] && $m == [lindex $args 3]} {
						if {$d >= [lindex $args 4] && $d <= [lindex $args 5]} {
							set found 1
						}
					}
				}
				month {
					#puts "$y $m - [lindex $args 1] [lindex $args 2]"
					if {$y == [lindex $args 1] && $m == [lindex $args 2]} {
						set found 1
					}
				}
			}
			if $found {
				$m_listbox insert end [format "%-16s:%4d:%s" $f $l [string range $s 0 32]]
			}
		}
	}
	
	method lookAt { } {
		set item [$m_listbox get [$m_listbox curselection]]
		set fi [split $item :]
		set file [lindex $fi 0]
		set line [lindex $fi 1]
		regsub -all {(\s*)} $file "" file
		regsub -all {(\s*)} $line "" line
		regsub -all {.txt} $file "" file
		#puts "file=$file, line=$line"
		
		$m_handler lookat $file $line
	}
	
	method resize { } {
		set w [winfo width  $m_canvas]
		set h [winfo height $m_canvas]
		#puts "$w $h"
		set uw [expr ($w-40)/30]
		set uh [expr ($h-20)/8]
		$m_canvas delete days
		for {set row 0} {$row < 8} {incr row} {
			for {set col 0} {$col < 30} {incr col} {
				set x1 [expr 20+$col*$uw]
				set y1 [expr 10+$row*$uh]
				set x2 [expr $x1+$uw]
				set y2 [expr $y1+$uh]
				$m_canvas create rectangle $x1 $y1 $x2 $y2 -fill white -outline black -tags "days cell_x${col}_y${row}"
			}
		}
	}
	
	method cache { cmd } {
		#puts "cache $cmd"
		switch $cmd {
			refresh {
				set lineNo 1
				set cache {}
				foreach f [glob -nocomplain "data/*.txt"] {
					puts $f
					
					set fd [open $f]
					set buf [read $fd]
					close $fd
					
					set lineNo 1
					foreach line [split $buf \n] {
						set found 0
						if {[regexp {(DATE:|TODO:|WAIT:|PLAN:)\s*\{\[([0-9]+)/0*([0-9]+)/0*([0-9]+)\s*.*\].*\}} $line tmp type y m d] > 0} {
							regsub .txt [file tail $f] "" f
							lappend cache [list $y $m $d $f $lineNo [string range $line 0 32]]
						}
						incr lineNo
					}
				}
				set fd [open data/date.cache w]
				puts $fd $cache
				close $fd
				set m_cache $cache
				[self] cache redraw
			}
			load {
				if ![file exists data/date.cache] {
					return
				}
				set fd [open data/date.cache]
				set m_cache [read $fd]
				close $fd
			}
			redraw {
				$m_calendar mark clear
				foreach c $m_cache {
					set y [lindex $c 0]
					set m [lindex $c 1]
					set d [lindex $c 2]
					#puts "$y $m $d"
					$m_calendar mark add $y $m $d
				}
				#$m_calendar redraw
			}
		}
	}
}

set app [Ko2NoteBookApp new]
$app add [Ko2NoteViewText new main]
